module Manager.ModalManager exposing (..)
-- Own
import Util.Util         exposing (MousePosition)
import Util.ShapeManager exposing (..)

type Modal
  = NoModal
  | ShowTable
  | ShowInput Shape String String
  | ShowShape Int

type ModalMsg
  = CloseModal
  -- show Modal
  | OpenShowTable
  | OpenShowInput
  | OpenShowShape Int
  -- for input / delete Shapes
  | ResetShowInput
  | Check String
  | Input Char String
  | InputPoint

-- Update
updateModal : ModalMsg -> Modal -> Modal
updateModal msg modal =
  case msg of
    CloseModal  -> NoModal
    OpenShowTable    -> if (eq_String_Modal "ShowTable" modal)
                          then NoModal
                          else ShowTable
    OpenShowInput    -> if (eq_String_Modal "ShowInput" modal)
                          then NoModal
                          else ShowInput Empty "" ""
    OpenShowShape id -> if (id <= 0)
                          then NoModal
                          else ShowShape id
    ResetShowInput -> ShowInput
                        (deleteCoordsFromShape
                          (getInputShapeFromModal modal)
                        ) "" ""
    Check shape -> if (eq_String_Modal "ShowInput" modal)
                     then if (eq_String_Shape shape (getInputShapeFromModal modal))
                            then ShowInput Empty                           "" ""
                            else ShowInput (shapeDefaultConstruktor shape) "" ""
                     else modal
    Input inputType value ->
      case modal of
        NoModal         -> modal -- ERROR
        ShowTable       -> modal -- ERROR
        ShowShape _     -> modal -- ERROR
        ShowInput iShape ix iy -> updateInput inputType value modal "ShowInput" iShape ix iy {x = 0, y = 0}
    InputPoint -> case modal of
                    NoModal           -> modal -- ERROR
                    ShowTable         -> modal -- ERROR
                    ShowShape _       -> modal -- ERROR
                    ShowInput iShape ix iy -> ShowInput (Polygon ((ix,iy)::(getPointListFromPolygon iShape))) "" ""
updateInput : Char -> String -> Modal -> String -> Shape -> String -> String -> MousePosition -> Modal
updateInput inputType value modal modalType iShape ix iy mouse =
    case inputType of
      '1' -> case iShape of
        Empty              -> modal -- ERROR
        Circle     _ b c   -> modalInputConstruktor modalType (Circle    value b c  ) ix iy mouse
        Rectangle  _ b c d -> modalInputConstruktor modalType (Rectangle value b c d) ix iy mouse
        Ellipse  _ b c d -> modalInputConstruktor modalType (Ellipse value b c d) ix iy mouse
        Polygon    _       -> modalInputConstruktor modalType iShape               value iy mouse
      '2' -> case iShape of
        Empty              -> modal -- ERROR
        Circle     a _ c   -> modalInputConstruktor modalType (Circle    a value c  ) ix iy mouse
        Rectangle  a _ c d -> modalInputConstruktor modalType (Rectangle a value c d) ix iy mouse
        Ellipse  a _ c d -> modalInputConstruktor modalType (Ellipse a value c d) ix iy mouse
        Polygon    _       -> modalInputConstruktor modalType iShape               ix value mouse
      '3' -> case iShape of
        Empty              -> modal -- ERROR
        Circle     a b _   -> modalInputConstruktor modalType (Circle    a b value  ) ix iy mouse
        Rectangle  a b _ d -> modalInputConstruktor modalType (Rectangle a b value d) ix iy mouse
        Ellipse  a b _ d -> modalInputConstruktor modalType (Ellipse a b value d) ix iy mouse
        Polygon    _       -> modal -- ERROR
      '4' -> case iShape of
        Empty              -> modal -- ERROR
        Circle     _ _ _   -> modal -- ERROR
        Rectangle  a b c _ -> modalInputConstruktor modalType (Rectangle a b c value) ix iy mouse
        Ellipse  a b c _ -> modalInputConstruktor modalType (Ellipse a b c value) ix iy mouse
        Polygon    _       -> modal -- ERROR
      _   -> modal -- ERROR
-- Modal Construktors
modalInputConstruktor : String -> Shape -> String -> String -> MousePosition -> Modal
modalInputConstruktor modalType shape x y mouse =
    case modalType of
      "ShowInput" -> ShowInput shape x y
      _           -> NoModal
-- Operations on Modal
eq_String_Modal : String -> Modal -> Bool
eq_String_Modal string modal = string==(modalNameToString modal)
-- Get Methods : Parameter of Modal
getInputShapeFromModal : Modal -> Shape
getInputShapeFromModal modal =
    case modal of
      NoModal             -> Empty
      ShowTable           -> Empty
      ShowInput shape _ _ -> shape
      ShowShape _         -> Empty
-- Transform Modal to other Types
modalNameToString : Modal -> String
modalNameToString modal =
    case modal of
      NoModal         -> "NoModal"
      ShowTable       -> "ShowTable"
      ShowInput _ _ _ -> "ShowInput"
      ShowShape _     -> "ShowShape"