module Manager.ViewManager exposing (view)
-- Html
import Html            exposing (Html, Attribute, div, text, node, button, a, h1, h2, label, figure, input, td, th, tr, table)
import Html.Attributes exposing (class, style, href, rel, id, title, value)
import Html.Events     exposing (onClick, onInput)
-- Svg
import Svg            exposing (Svg,svg, image,rect,ellipse,circle,polygon,g)
import Svg.Attributes exposing (width,height, viewBox, version, xlinkHref, x, y, cx, cy,rx,ry, r)
import Svg.Events
-- Own
import TypeHolder exposing (Model,Msg(..))
import Manager.StateManager exposing (..)
import Manager.ModalManager exposing (Modal(..), ModalMsg(..))
import Util.ShapeManager exposing (..)
import Util.Util         exposing (..)

-- VIEW
view : Model -> Html Msg
view model =
  div [style "background-color" "#fff"]
      (List.concat
        [[ link [ rel "stylesheet" -- Bluma
                , href "https://cdn.jsdelivr.net/npm/bulma@0.8.0/css/bulma.min.css"
                ] []
         , link [ rel "stylesheet" -- Animationen
                , href "https://cdnjs.cloudflare.com/ajax/libs/animate.css/3.7.2/animate.min.css"
                ] []
         , link [ rel "stylesheet" -- Icon für DropdownMenue
                , href "https://use.fontawesome.com/releases/v5.8.1/css/all.css"
                ] []
         , viewHeader_Buttons model.state
         , Html.section [class "section"] []
         , viewImage model
         , case model.shapesLoadResult of
             Ok  s -> text ""
             Err e -> text ("Could not load shapes: "++(httpErrorToString e))
         ]
        , case (getModalFromState model.state) of
            NoModal                -> []
            ShowInput iShape ix iy -> [viewNewShapeInputModal iShape ix iy]
            ShowTable              -> [viewTableModal         model.saved ]
            ShowShape id           -> [viewShapeModal         id          ]
        ]
      )
-- Parts of the VIEW
viewHeader_Buttons : State -> Html Msg
viewHeader_Buttons state =
    div []
        [ div [class "section"]
              [ h1 [class "title"] [text "Formen über der Informatik"]
              , h2 [class "subtitle"] [text "Eingeben, Anzeigen, Finden und Clicken"]
              ]
        , div [ style "margin-left" "25px"]
              [ button [ onClick (ManageModal OpenShowInput "")
                       , class "button is-success is-pulled-left is-size-6"
                       , style "margin-bottom" "15px"
                       , style "margin-right" "15px"
                       ] [ text "Neue Form hinzüfügen" ]
              , button [ onClick (ManageModal OpenShowTable "")
                       , class "button is-success is-pulled-left is-size-6"
                       , style "margin-right" "15px"
                       , style "margin-bottom" "15px"
                       ] [ text "Tabelle mit Formen zeigen" ]
              , button [ onClick (LoadShapes (Ok "LoadExample"))
                       , class "button is-success is-pulled-left is-size-6"
                       , style "margin-right" "15px"
                       , style "margin-bottom" "15px"
                       , title "Achtung: aktuelle Shapes gehen dabei verloren!"
                       ] [ text "Beispiel mit Formen laden" ]
              , viewDropDownMenue "Objekte mit Formen hervorheben" True "angle-down" []
                  [ a [ class "dropdown-item"
                      , href "#Fenster"
                      ] [text "Lade Fenster"]
                  , a [class "dropdown-item"
                      , href "#Türen"
                      ] [text "Lade Türen"]
                  ]
              , if (eq_String_State "DrawShape" state)
                  then let iShape = getInputShapeFromState state
                       in if (eq_String_Shape "Polygon" iShape)
                            then viewDropDownMenue "Polygon wird gezeichnet" True "angle-down" []
                                   [ a [class "dropdown-item"
                                       , onClick (ManageShapes (AddShape Empty))
                                       , href ""
                                       ] [text "Speichern"]
                                   , a [class "dropdown-item"
                                       , onClick (ManageModal CloseModal "")
                                       , href ""
                                       ] [text "Abbrechen"]
                                   ]
                            else viewDropDownMenue
                                   (shapeNameStringToGerman
                                     (shapeNameToString
                                       (getInputShapeFromState state))++" zeichnen: abbrechen  ")
                                   False "times" [onClick (ManageModal CloseModal "")] []
                  else viewDropDownMenue "Neue Form zeichnen" True "angle-down" []
                         [ a [class "dropdown-item"
                             , onClick (OpenDrawShape (Circle "" "" ""))
                             , href ""
                             ] [text "Kreis"]
                         , a [class "dropdown-item"
                             , onClick (OpenDrawShape (Rectangle "" "" "" ""))
                             , href ""
                             ] [text "Rechteck"]
                        , a [class "dropdown-item"
                           , onClick (OpenDrawShape (Ellipse "" "" "" ""))
                           , href ""
                           ] [text "Ellipse"]
                         , a [class "dropdown-item"
                             , onClick (OpenDrawShape (Polygon []))
                             , href ""
                             ] [text "Vieleck"]
                         ]
              ]
        ]
viewImage : Model -> Html Msg
viewImage model =
    div [ style "margin-left" "25px", style "margin-top" "15px", class "container"]
        [ figure [ class "image"]
                 [ svg [ width "100%", viewBox "0 0 800 600", version "1.1"]
                       ( List.concat
                           [[image [ width "800", height "600"
                                   , xlinkHref "http://www.informatik.uni-halle.de/im/1285058520_1381_00_800.jpg"
                                   ] []
                            ]
                           , viewSvgShapes model.saved model.svgShapeHover 1
                           , case model.state of
                               Standby         -> []
                               ActiveModal _   -> []
                               DrawShape iShape mPos ->
                                 [ case iShape of
                                     Empty             -> rect [] []
                                     Circle    x y rad ->
                                       if x=="" then circle [cx (String.fromInt mPos.x)
                                                            , cy (String.fromInt mPos.y), r "5"] []
                                                else circle [cx x, cy y, r (abstand x y mPos)] []
                                     Ellipse a b w h ->
                                       if a=="" then ellipse [ cx (String.fromInt mPos.x)
                                                          , cy (String.fromInt mPos.y)
                                                          , rx "10", ry "10"] []
                                                else let coords = manageEllipseCoords
                                                                    {x= a
                                                                    ,y= b
                                                                    ,w= (abstand1D mPos.x (stringToInt a))
                                                                    ,h= (abstand1D mPos.y (stringToInt b))
                                                                    }
                                                     in  ellipse [ cx      coords.x
                                                              , cy      coords.y
                                                              , rx  coords.w
                                                              , ry coords.h
                                                              ] []
                                     Rectangle a b w h ->
                                       if a=="" then rect [ x (String.fromInt mPos.x)
                                                          , y (String.fromInt mPos.y)
                                                          , width "10", height "10"] []
                                                else let coords = manageRectCoords
                                                                    {x= a
                                                                    ,y= b
                                                                    ,w= (abstand1D mPos.x (stringToInt a))
                                                                    ,h= (abstand1D mPos.y (stringToInt b))
                                                                    }
                                                     in  rect [ x      coords.x
                                                              , y      coords.y
                                                              , width  coords.w
                                                              , height coords.h
                                                              ] []
                                     Polygon   pList   ->
                                       case pList of
                                         []    -> circle   [ cx (String.fromInt mPos.x)
                                                           , cy (String.fromInt mPos.y), r "5"
                                                           ] []
                                         p::[] -> Svg.line [ Svg.Attributes.x1 (Tuple.first  p)
                                                           , Svg.Attributes.y1 (Tuple.second p)
                                                           , Svg.Attributes.x2 (String.fromInt mPos.x)
                                                           , Svg.Attributes.y2 (String.fromInt mPos.y)
                                                           , Svg.Attributes.style "stroke:#000;stroke-width:2"
                                                           ] []
                                         p::ps -> polygon [ Svg.Attributes.points
                                                            (shapeCoordsToHtmlCoords
                                                              (Polygon (( String.fromInt mPos.x
                                                                        , String.fromInt mPos.y
                                                                        )::pList
                                                                       )
                                                              )
                                                            )
                                                          ] []
                                 , rect [ x "0", y "0", width "800", height "600"
                                        , Svg.Attributes.style
                                            "fill:blue;stroke:pink;stroke-width:5;fill-opacity:0.1;stroke-opacity:0.9"
                                        , onMouseMove UpdateMousePos
                                        , Svg.Events.onClick InputDrawShape
                                        ] []
                                 ]
                           ]
                       )

                 ]
        ]
viewNewShapeInputModal : Shape -> String -> String -> Html Msg
viewNewShapeInputModal iShape ix iy =
    modalCard
      [ modalHeader "Neue Form hinzufügen"
      , modalCardBody
          (List.append
            [ viewDropDownMenue (if (shapeNameToString iShape)==""
                                   then "Form auswählen"
                                   else (shapeNameStringToGerman(shapeNameToString iShape))
                                ) False "angle-down" []
                [ a [ class "dropdown-item"
                    , onClick (ManageModal (Check "Circle") "")
                    , href ""
                    ] [text "Kreis"]
                , a [ class "dropdown-item"
                    , onClick (ManageModal (Check "Rectangle") "")
                    , href ""
                    ] [text "Rechteck"]
                , a [ class "dropdown-item"
                    , onClick (ManageModal (Check "Ellipse") "")
                    , href ""
                    ] [text "Ellipse"]
                , a [ class "dropdown-item"
                    , onClick (ManageModal (Check "Polygon" ) "")
                    , href ""
                    ] [text "Vieleck"]
                ]
            , if (shapeNameToString iShape)==""
                then div [style "margin-top" "120px"] []
                else div [] []
            ]
            (case iShape of
             Empty             -> []
             Circle    a b c   ->
               [ div []
                     [ label [class "label"] [text "X-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "X-Position"
                                        , value a, onInput (ManageModal (Input '1' "")) ] [] ]
                     , label [class "label"] [text "Y-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Y-Position"
                                        , value b, onInput (ManageModal (Input '2' "")) ] [] ]
                     , label [class "label"] [text "Radius"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Radius"
                                        , value c, onInput (ManageModal (Input '3' "")) ] [] ]
                     ]
               ]
             Rectangle a b c d ->
               [ div []
                     [ label [class "label"] [text "X1-Position-"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "X1-Position"
                                        , value a, onInput (ManageModal (Input '1' "")) ] [] ]
                     , label [class "label"] [text "Y1-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Y1-Position"
                                        , value b, onInput (ManageModal (Input '2' "")) ] [] ]
                     , label [class "label"] [text "X2-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "X2-Position"
                                        , value c, onInput (ManageModal (Input '3' "")) ] [] ]
                     , label [class "label"] [text "Y2-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Y2-Position"
                                        , value d, onInput (ManageModal (Input '4' "")) ] [] ]
                     ]
               ]
             Ellipse a b c d ->
               [ div []
                     [ label [class "label"] [text "X1-Position-"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "X1-Position"
                                        , value a, onInput (ManageModal (Input '1' "")) ] [] ]
                     , label [class "label"] [text "Y1-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Y1-Position"
                                        , value b, onInput (ManageModal (Input '2' "")) ] [] ]
                     , label [class "label"] [text "X2-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "X2-Position"
                                        , value c, onInput (ManageModal (Input '3' "")) ] [] ]
                     , label [class "label"] [text "Y2-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Y2-Position"
                                        , value d, onInput (ManageModal (Input '4' "")) ] [] ]
                     ]
               ]
             Polygon   _       ->
               [ div []
                     [ label [class "label"] [text "X-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "X-Position"
                                        , value ix, onInput (ManageModal (Input '1' "")) ] [] ]
                     , label [class "label"] [text "Y-Position"]
                     , label [] [ input [class "input", Html.Attributes.placeholder "Y-Position"
                                        , value iy, onInput (ManageModal (Input '2' "")) ] [] ]
                     , label [class "label"] [text "Eingegebene Vieleck Koordinaten: "]
                     , text (shapeCoordsToString iShape)
                     , div [] [ button [ onClick (ManageModal InputPoint "")
                                       , class "button is-success is-size-7"
                                       , style "margin-top" "10px"
                                       ] [ text "Punkt hinzufügen" ]
                              ]
                     ]
               ]
            )
          )
      , modalFooter [ button [ class "button is-success"
                             , onClick (ManageShapes (AddShape Empty))
                             ] [text "Form hinzufügen"]
                    ] 27
      ]
viewTableModal : List (Shape) -> Html Msg
viewTableModal modelSaved =
    modalCard
      [ modalHeader "Tabelle mit Formen"
      , modalCardBody
          ( if List.length modelSaved == 0
              then [ div [] [text "Es sind keine Formen vorhanden!"] ]
              else [ table [class "table"]
                           ( List.append [ tr []
                                              [ th [] [text "ID"]
                                              , th [] [text "Form"]
                                              , th [] [text "Koordinaten"]
                                              , th [] [text ""]
                                              ]
                                         ]
                                         ( writeShapeToTable modelSaved 1)
                           )
                   ]
          )
      , modalFooter [] 0
      ]
viewShapeModal : Int -> Html Msg
viewShapeModal id =
    if id <= 0 then div [] [] -- ERROR
               else modalCard
                      [ modalHeader ("Form "++String.fromInt id)
                      , Html.section [ class "modal-card-body"]
                                     [ text ("Form "++String.fromInt id++" wurde gefunden!")]
                      , modalFooter [] 0
                      ]
-- Helping Methods for the VIEW
viewDropDownMenue : String -> Bool -> String -> List(Attribute Msg) -> List(Svg Msg) -> Html Msg
viewDropDownMenue header isGreen iconType attributes dropDownItems =
  div (List.append
        [ class "dropdown is-hoverable"
        , style "margin-right" "15px"
        , style "margin-bottom" "15px"
        ] attributes
      )
      [ div [ class "dropdown-trigger"]
            [ button [ class ("button "++(if isGreen then "is-success" else "")
                                       ++" aria-haspopup=true aria-controls=dropdown-menu")
                     ]
                     [ Html.span [] [text header]
                     , Html.span [ class "icon is-small"]
                                 [ Html.i [class ("fas fa-"++iconType++" aria-hidden=true")] [] ]
                     ]
            ]
      , if dropDownItems==[]
          then div [] []
          else div [ class "dropdown-menu role=menu"]
                   [ div [ class "dropdown-content"] dropDownItems ]
      ]
viewSvgShapes : List Shape -> Int -> Int -> List (Svg Msg)
viewSvgShapes shapes activeShape id =
    case shapes of
      []    -> []
      x::xs -> ( case x of
                   Empty               -> rect [] []
                   Circle    a b c     -> circle ([cx a, cy b, r c                                  ]
                                                  ++svgShapeStyle activeShape id) []
                   Rectangle a b c d   -> rect   ([Svg.Attributes.x a, y b, width c, height d       ]
                                                  ++svgShapeStyle activeShape id) []
                   Ellipse a b c d   -> ellipse   ([Svg.Attributes.cx a, cy b, rx c, ry d       ]
                                                  ++svgShapeStyle activeShape id) []
                   Polygon   pointList -> polygon([Svg.Attributes.points (shapeCoordsToHtmlCoords x)]
                                                  ++svgShapeStyle activeShape id) []
               )::viewSvgShapes xs activeShape (id+1)
svgShapeStyle : Int -> Int -> List( Svg.Attribute Msg)
svgShapeStyle activeShape id =
    [ Svg.Attributes.fill "white"
    , Svg.Attributes.fillOpacity (if activeShape==id then "0.5" else "0.0")
    , Svg.Attributes.stroke "red"
    , Svg.Attributes.strokeWidth "3"
    , Svg.Events.onMouseOver (SvgShapeHover id)
    , Svg.Events.onMouseOut  (SvgShapeHover 0)
    , Svg.Events.onClick     (ManageModal (OpenShowShape id) "")
    ]
writeShapeToTable : List ( Shape ) -> Int -> List ( Html Msg )
writeShapeToTable shapeList id =
    case shapeList of
      []    -> []
      x::xs -> tr [Html.Attributes.id ("Shape"++String.fromInt id)]
                  [ td [] [text ("Form"++String.fromInt id)]
                  , td [] [text (shapeNameStringToGerman(shapeNameToString        x))]
                  , td [] [text (shapeCoordsToString      x)]
                  , td [] [button [ class "button is-danger is-size-7"
                                  , onClick (ManageShapes (DeleteShape id))] [ text "Form löschen" ] ]
                  ]
               :: (writeShapeToTable xs (id+1))
-- Modal view-Elements
modalCard : List (Html Msg) -> Html Msg
modalCard modalParts =
    div [class "modal is-active"]
        [ div [class "modal-background"] []
        , div [class "modal-card"] modalParts
        ]
modalHeader : String -> Html Msg
modalHeader title = Html.header [class "modal-card-head"]
                                [ Html.p [class "modal-card-title" ] [text title]
                                , button [class "delete aria-label=close", onClick (ManageModal CloseModal "")] []
                                ]
modalCardBody : List (Html Msg) -> Html Msg
modalCardBody bodyElements = Html.section [class "modal-card-body"] bodyElements
modalFooter : List (Html Msg) -> Int-> Html Msg
modalFooter footElements footElementsWidth =
    Html.footer [class "modal-card-foot"]
                (List.append footElements
                            [ button [ class "button"
                                     , style "margin-left" ((String.fromInt (83-footElementsWidth))++"%")
                                     , onClick (ManageModal CloseModal "")
                                     ][text "Schließen"]
                            ]
                )