module Util.ShapeManager exposing (..)
-- Json
import Json.Decode as JD

type Shape
  = Empty
  | Circle String String String
  | Rectangle String String String String
  | Ellipse   String String String String
  | Polygon (List ( String, String ) )

type ShapesMsg
  = DeleteShape Int
  | AddShape Shape

-- Update
updateShapes : ShapesMsg -> List Shape -> List Shape
updateShapes shapesMsg shapeList =
    case shapesMsg of
      DeleteShape id -> List.append (List.take (id-1) shapeList)
                                    (List.drop (id)   shapeList)
      AddShape shape -> shape::shapeList

-- Shape Construktors
shapeDefaultConstruktor : String -> Shape
shapeDefaultConstruktor shape =
    case shape of
      "Circle"   -> Circle    "" "" ""
      "Rectangle"-> Rectangle "" "" "" ""
      "Ellipse"  -> Ellipse "" "" "" ""
      "Polygon"  -> Polygon   []
      _          -> Empty
shapesDecoder : String -> List Shape
shapesDecoder s =
    if (String.contains "shapes" s)
      then []
      else
        List.concat
          [ if (String.contains "circle" s)
              then [ Circle (shapesDecoderList "circle" "x" s)
                            (shapesDecoderList "circle" "y" s)
                            (shapesDecoderList "circle" "r" s)
                   ]
              else []
          , if (String.contains "rectangle" s)
              then [ Rectangle (shapesDecoderList "rectangle" "x"      s)
                               (shapesDecoderList "rectangle" "y"      s)
                               (shapesDecoderList "rectangle" "width"  s)
                               (shapesDecoderList "rectangle" "height" s)
                   ]
              else []
            , if (String.contains "Ellipse" s)
            then [ Ellipse (shapesDecoderList "Ellipse" "cx"      s)
                             (shapesDecoderList "Ellipse" "cy"      s)
                             (shapesDecoderList "Ellipse" "rx"  s)
                             (shapesDecoderList "Ellipse" "ry" s)
                 ]
            else []
          , if (String.contains "polygon" s)
              then [ Polygon (List.map2 Tuple.pair
                             (shapesDecoderListPoly "x" s)
                             (shapesDecoderListPoly "y" s)
                             )
                   ]
              else []
          ]
-- Operations on Shapes
eq_String_Shape : String -> Shape -> Bool
eq_String_Shape string shape = string==(shapeNameToString shape)
deleteCoordsFromShape : Shape -> Shape
deleteCoordsFromShape shape =
    case shape of
      Empty             -> Empty
      Circle    _ _ _   -> Circle    "" "" ""
      Rectangle _ _ _ _ -> Rectangle "" "" "" ""
      Ellipse   _ _ _ _ -> Ellipse "" "" "" ""
      Polygon   _       -> Polygon   []
-- Get Methods : Parameter of Shapes
getPointListFromPolygon : Shape -> List (String,String)
getPointListFromPolygon shape =
    case shape of
      Empty               -> []
      Circle    _ _ _     -> []
      Rectangle _ _ _ _   -> []
      Ellipse _ _ _ _   -> []
      Polygon   pointList -> pointList
-- Transform Shape to other Types
shapeNameToString : Shape -> String
shapeNameToString shape =
    case shape of
      Empty             -> ""
      Circle    _ _ _   -> "Circle"
      Rectangle _ _ _ _ -> "Rectangle"
      Ellipse _ _ _ _ -> "Ellipse"
      Polygon   _       -> "Polygon"
shapeNameStringToGerman : String -> String
shapeNameStringToGerman shape =
    case shape of
      "Circle"   -> "Kreis"
      "Rectangle"-> "Rechteck"
      "Polygon"  -> "Vieleck"
      "Ellipse"  -> "Ellipse"
      _          -> ""
shapeCoordsToString : Shape -> String
shapeCoordsToString shape =
    case shape of
      Empty                                       -> ""
      Circle    x_center y_center radius          -> x_center++","++y_center++","++radius
      Rectangle x_left   y_upper  x_right y_lower -> x_left++","++y_upper++","++x_right++","++y_lower
      Ellipse   x_left   y_upper  x_right y_lower -> x_left++","++y_upper++","++x_right++","++y_lower
      Polygon   pointList                         ->
          case pointList of
            []        -> ""
            (x,y)::xs -> "("++x++","++y++") "++(shapeCoordsToString (Polygon xs))
shapeCoordsToHtmlCoords : Shape -> String
shapeCoordsToHtmlCoords shape =
    case shape of
      Empty             -> ""
      Circle    _ _ _   -> shapeCoordsToString shape
      Rectangle _ _ _ _ -> shapeCoordsToString shape
      Ellipse   _ _ _ _ -> shapeCoordsToString shape
      Polygon   _       -> String.left ((String.length (shapeCoordsToHtmlCoordsHelper shape))-1)
                                       (shapeCoordsToHtmlCoordsHelper shape)
shapeCoordsToHtmlCoordsHelper : Shape -> String
shapeCoordsToHtmlCoordsHelper shape =
    case shape of
      Empty               -> ""
      Circle    _ _ _     -> shapeCoordsToString shape
      Rectangle _ _ _ _   -> shapeCoordsToString shape
      Ellipse   _ _ _ _   -> shapeCoordsToString shape
      Polygon   pointList ->
          case pointList of
            []        -> ""
            (x,y)::xs -> x++","++y++","++(shapeCoordsToHtmlCoordsHelper (Polygon xs))
-- Decoder of shapes
rectListDecoder : JD.Decoder (List Shape)
rectListDecoder = JD.field "rectangle" (JD.list (JD.map4
                                                  Rectangle
                                                    (JD.field "x"      JD.string)
                                                    (JD.field "y"      JD.string)
                                                    (JD.field "width"  JD.string)
                                                    (JD.field "height" JD.string)
                                                )
                                       )
ellipseListDecoder : JD.Decoder (List Shape)
ellipseListDecoder = JD.field "ellipse" (JD.list (JD.map4
                                                  Ellipse
                                                    (JD.field "cx"      JD.string)
                                                    (JD.field "cy"      JD.string)
                                                    (JD.field "rx"  JD.string)
                                                    (JD.field "ry" JD.string)
                                                )
                                       )
shapesDecoderList : String -> String -> String -> String
shapesDecoderList shape var s =
    let res = JD.decodeString
                (JD.field shape (JD.list (JD.field var JD.string)))
                s
    in case res of
         Ok stringList -> case stringList of
                            []       -> ""
                            x::[]    -> x
                            x::xs    -> x++" This List is too long"
         Err e         -> "ERROR"
shapesDecoderListPoly : String -> String -> List String
shapesDecoderListPoly var s =
    let res = JD.decodeString
                (JD.field "polygon" (JD.list (JD.field "points" (JD.list (JD.field var JD.string)))))
                s
    in case res of
         Ok list -> case list of
                      []    -> []
                      x::[] -> x
                      x::xs -> x++["This List is too long"]
         Err e   -> ["ERROR"]